﻿local DEFAULT_LANGUAGE = GetDefaultLanguage("player")

local R, RR = 1, 2 -- random, round robin
local selectedID, selectedMsg, selectedChan, selectedRotation = 0, 0, 0, 0
local nextTTL = GetTime()
local supportedChannels = {}
local ORG_ChatEdit_InsertLink = nil
local ORG_SetItemRef = nil
local isBG = false
local TimeSinceLastUpdate = 0

AB_UpdateInterval = 1.0
AB_EditBoxInFocus = nil
AB_Data = {}
AB_On = true
AB_Variables = {}

function AB_OnLoad()
	TimeSinceLastUpdate = 0
	ABHiddenFrame:RegisterEvent("ADDON_LOADED")
	ABHiddenFrame:RegisterEvent("PLAYER_LOGIN")
	-- Set Up Hooks
	ORG_ChatEdit_InsertLink = ChatEdit_InsertLink
	ChatEdit_InsertLink = AB_ChatEdit_InsertLink
	ORG_SetItemRef = SetItemRef
	SetItemRef = AB_SetItemRef
end

function AB_OnEvent(frame,event,arg_1,arg_2)
	if event == "CHAT_MSG_SKILL" then
		AB_Skill_OnUpdate()
	elseif event == "ZONE_CHANGED" or event == "ZONE_CHANGED_NEW_AREA" then
		Is_BG = false
		local currentZoneName = GetZoneText()
		for i=1, #(AB_BG) do
			if (currentZoneName == AB_BG[i]) then
				Is_BG = true
				break
			end
		end
	elseif event == "ADDON_LOADED" and arg_1 == "AB" then
		SlashCmdList["AB"] = AB_SlashHandler
		SLASH_AB1 = "/ab"
		ABDropDownMenu.displayMode = "MENU"
		ABChanDDM.displayMode = "MENU"
		if #(AB_Variables) == 0 then
			AB_ResetVariables()
		end
	elseif event == "TRADE_SKILL_SHOW" then
		if ATSWSkillIcon then
			ATSWSkillIcon:SetScript("OnClick", AB_ATSW_SkillIcon_OnClick)
		end
	elseif event == "PLAYER_LOGIN" then
		AB_Activate(AB_On)
	end
end

function AB_SlashHandler(msg)
	local words = {}
	for word in string.gmatch(msg, "%w+") do
		tinsert(words, word)
	end
	if #(words) > 0 then
		local cmd = strlower(words[1])
		if cmd == "on" then
			AB_Activate(true)
		elseif cmd == "off" then
			AB_Activate(false)
		elseif cmd == "round" then
			if words[2] then
				AB_SetRotation(words[2], RR)
			end
		elseif cmd == "random" then
			if words[2] then
				AB_SetRotation(words[2], R)
			end
		elseif cmd == "remove" or cmd == "r" then
			if words[3] and tonumber(words[3]) then
				AB_RemoveMsg(words[2],tonumber(words[3]))
			elseif words[2] then
				AB_Remove(words[2])
			end
		elseif cmd == "send" or cmd == "s" then
			if words[2] then
				AB_Send(words[2])
			end
		elseif cmd == "chan" or cmd == "c" then
			if #(words) > 2 then
				local chan = AB_Strip(msg, words[1].." "..words[2].." ")
				if chan ~= nil and chan ~= "" then
					AB_Insert(words[2], nil, nil, chan, nil, nil, nil, nil, nil)
				end
			end
		elseif cmd == "interval" or cmd == "i" then
			if #(words) == 3 and tonumber(words[3]) then
				AB_Insert(words[2], nil, nil, nil, nil, tonumber(words[3]), nil, nil, nil)
			end
		elseif cmd == "msg" or cmd == "m" then
			if #(words) > 2 then
				local msgindex = nil
				local message = nil
				if words[3] and tonumber(words[3]) then
					message = AB_Strip(msg, words[1].." "..words[2].." "..words[3].." ")
					msgindex = tonumber(words[3])
				else
					message = AB_Strip(msg, words[1].." "..words[2].." ")
				end
				if message ~= nil and message ~= "" then
					AB_Insert(words[2], msgindex, message, nil, nil, nil, nil, nil, nil)
				end
			end
		elseif cmd == "edit" or cmd == "e" then
			if #(words) == 2 then
				AB_Edit(words[2],1)
			end
		elseif cmd == "new" or cmd == "n" then
			AB_New_Group()
		elseif cmd == "enable" then
			if #(words) == 2 then
				AB_Toggle(words[2], true)
			end
		elseif cmd == "disable" then
			if #(words) == 2 then
				AB_Toggle(words[2], false)
			end
		elseif cmd == "rv" or cmd == "resetvariables" then
			AB_ResetVariables()
		elseif cmd == "status" then
			if #(words) > 1 then
				AB_Status(words[2])
			else
				AB_Status(nil)
			end
		end
	else
		AB_Activate(not AB_On)
	end
end

function AB_Activate(state)
	if state then
		ABHiddenFrame:RegisterEvent("ZONE_CHANGED")
		ABHiddenFrame:RegisterEvent("ZONE_CHANGED_NEW_AREA")
		ABHiddenFrame:RegisterEvent("CHAT_MSG_SKILL")
		ABHiddenFrame:RegisterEvent("TRADE_SKILL_SHOW")
		AB_On = true
		AB_Print(AB_LABEL .. AB_ON)
		for i=1,#(AB_Data) do
			if AB_Data[i] and AB_Data[i].enabled == true and AB_Data[i].ttl > GetTime() then
				 AB_Data[i].ttl = GetTime() + mod(AB_Data[i].ttl,AB_Data[i].interval) + (i * 30)
			end
		end
		nextTTL = GetTime()
		AB_Skill_OnUpdate()
	else
		ABHiddenFrame:UnregisterEvent("ZONE_CHANGED")
		ABHiddenFrame:UnregisterEvent("ZONE_CHANGED_NEW_AREA")
		ABHiddenFrame:UnregisterEvent("CHAT_MSG_SKILL")
		ABHiddenFrame:UnregisterEvent("TRADE_SKILL_SHOW")
		AB_On = false
		AB_Print(AB_LABEL .. AB_OFF)
	end
	if IsAddOnLoaded("ABFu") then
		ABFu:Update()
	end
end

function AB_OnUpdate(self, elapsed)
	TimeSinceLastUpdate = TimeSinceLastUpdate + elapsed
	if AB_On and TimeSinceLastUpdate > AB_UpdateInterval and nextTTL <= GetTime() then
		for i=1,#(AB_Data) do
			if AB_Data[i] and AB_Data[i].enabled == true and AB_Data[i].ttl <= GetTime() and #(AB_Data[i].msg) > 0 and AB_Data[i].chan ~= "" then
				AB_Broadcast(i)
			end
		end
	end
end

function AB_Broadcast(index)
	if AB_IsPaused() then
		return
	end
	-- get message depending on rotation type
	local msgindex = 1
	if AB_Data[index].rotation == RR then
		msgindex = AB_Data[index].nextmsg
		if msgindex == #(AB_Data[index].msg) then
			AB_Data[index].nextmsg = 1
		else
			AB_Data[index].nextmsg = AB_Data[index].nextmsg + 1
		end
	elseif AB_Data[index].rotation == R then
		msgindex = math.random(#(AB_Data[index].msg))
	end
	local msg = AB_ReplaceVariables(AB_Data[index].msg[msgindex])

	local tmp = strupper(AB_Data[index].chan)
	local id = 0
	local channel = ""
	if tmp == "GUILD" or tmp == "SAY" or tmp == "YELL" or tmp == "PARTY" or tmp == "RAID" or tmp == "OFFICER" then
		channel = tmp
	else
		channel = "CHANNEL"
		if tonumber(tmp) then
			id = tonumber(tmp)
		else
			local id, name = GetChannelName(tmp)
			if (id > 0 and name ~= nil) then
				local type
				if AB_Data[index].password ~= "" then
					type, name = JoinChannelByName(name,AB_Data[index].password,DEFAULT_CHAT_FRAME)
				else
					type, name = JoinChannelByName(name,nil,DEFAULT_CHAT_FRAME)
				end
				if type == 0 then
					channel = ""
					id = 0
				end
			end
		end
	end
	if channel ~= "" then
		local msgs = { strsplit("\n", msg) }
		for i=1, #(msgs) do
			ChatThrottleLib:SendChatMessage("BULK", AB_MENU, msgs[i], channel, DEFAULT_LANGUAGE, id)
		end
	end
	
	AB_Data[index].ttl = AB_Data[index].interval + GetTime()
	if nextTTL > AB_Data[index].ttl then
		nextTTL = AB_Data[index].ttl
	end
end

function AB_SetRotation(id, rotation)
	if AB_Data[id] then
		AB_Data[id].rotation = rotation
		AB_Print(format(AB_STATUS_ID, data.id) .. " " .. AB_ROTATION[rotation])
	end
end

function AB_Insert(id, msgindex, msg, chan, password, interval, ttl, enabled, rotation)
	if id ~= nil and strtrim(id) ~= "" then
		id = strtrim(strupper(id))
		local updateIndex = 0
		local data = {
			["id"] = "",
			["msg"] = {},
			["chan"] = "",
			["password"] = "",
			["interval"] = 180,
			["ttl"] = GetTime(),
			["enabled"] = true,
			["rotation"] = RR,
			["nextmsg"] = 1
		}
		for i=1,#(AB_Data) do
			if AB_Data[i] and AB_Data[i].id == id then
				data = AB_Data[i]
				updateIndex = i
				break
			end
		end
		if rotation ~= nil and rotation ~= 0 and tonumber(rotation) then
			data.rotation = tonumber(rotation)
			AB_Print(format(AB_STATUS_ID, data.id) .. " " .. AB_ROTATION[rotation])
		end
		if msg ~= nil and msg ~= "" then
			if msgindex == 0 then
				tinsert(data.msg, msg)
				AB_Print(format(AB_STATUS_MSG, #(data.msg), msg))
			elseif data.msg[msgindex] ~= msg then
				data.msg[msgindex] = msg
				AB_Print(format(AB_STATUS_MSG, msgindex, msg))
			end
		end
		if chan ~= nil and chan ~= "" and data.chan ~= chan then
			data.chan = chan
			AB_Print(format(AB_STATUS_CHAN, data.chan))
		end
		if password ~= nil and password ~= "" and data.password ~= password then
			data.password = password
			AB_Print(format(AB_STATUS_PWD, data.password))
		end
		if interval ~= nil then
			interval = tonumber(interval) or 0
			if interval ~= 0 and data.interval ~= interval then
				data.interval = interval
				AB_Print(format(AB_STATUS_INTERVAL, data.interval))
			end
		end
		if ttl ~= nil and tonumber(ttl) then
			data.ttl = ttl
			AB_Print(format(AB_STATUS_TTL, data.ttl))
		end
		if enabled ~= nil and enabled ~= data.enabled then
			if enabled == true then
				data.enabled = true
				data.ttl = GetTime()
				AB_Print(format(AB_STATUS_STATE,AB_TRUE))
			else
				data.enabled = false
				AB_Print(format(AB_STATUS_STATE,AB_FALSE))
			end
		end
		if updateIndex == 0 then
			data.id = id
			tinsert(AB_Data, data)
		else
			AB_Data[updateIndex] = data
		end
		if IsAddOnLoaded("ABFu") then
			ABFu:UpdateData()
		end
	end
end

function AB_Remove(x)
	local id = strtrim(strupper(x))
	for i=1,#(AB_Data) do
		if AB_Data[i] and AB_Data[i].id == id then
			AB_Print(format(AB_STATUS_REMOVE,AB_Data[i].id))
			tremove(AB_Data, i)
			if IsAddOnLoaded("ABFu") then
				ABFu:Update()
			end
			break
		end
	end
end

function AB_RemoveMsg(x, y)
	local id = strtrim(strupper(x))
	for i=1,#(AB_Data) do
		if AB_Data[i] and AB_Data[i].id == id then
			AB_Print(format(AB_STATUS_REMOVEMSG,AB_Data[i].msg[y]))
			tremove(AB_Data[i].msg, y)
			if IsAddOnLoaded("ABFu") then
				ABFu:Update()
			end
			break
		end
	end
end

function AB_RemoveCurr()
	if AB_Data[selectedID] and AB_Data[selectedID].msg[selectedMsg] then
		AB_Print(format(AB_STATUS_REMOVEMSG,AB_Data[selectedID].msg[selectedMsg]))
		tremove(AB_Data[selectedID].msg, selectedMsg)
		if #(AB_Data[selectedID].msg) > 0 then
			AB_Edit(AB_Data[selectedID].id, #(AB_Data[selectedID].msg))
		else
			AB_New_Group()
		end
		if IsAddOnLoaded("ABFu") then
			ABFu:Update()
		end
	end
end

function AB_Toggle(x, state)
	local id = strtrim(strupper(x))
	for i=1,#(AB_Data) do
		if AB_Data[i] and AB_Data[i].id == id then
			if state == true then
				AB_Data[i].enabled = true
				AB_Data[i].ttl = GetTime()
				AB_Print(format(AB_STATUS_STATE,AB_TRUE))
			else
				AB_Data[i].enabled = false
				AB_Print(format(AB_STATUS_STATE,AB_FALSE))
			end
			break
		end
	end
end

function AB_Send(x)
	local id = strtrim(strupper(x))
	for i=1,#(AB_Data) do
		if AB_Data[i] and AB_Data[i].id == id and AB_Data[i].chan ~= "" and AB_Data[i].msg ~= "" then
			AB_Broadcast(i)
		end
	end
end

function AB_Status(x)
	local id = x
	if id ~= nil then
		id = strtrim(strupper(id))
	end
	for i=1,#(AB_Data) do
		if AB_Data[i] and (id == nil or id == AB_Data[i].id) then
			AB_Print(AB_STATUS_SEPARATOR)
			AB_Print(format(AB_STATUS_ID,AB_Data[i].id))
			for x=1,#(AB_Data[i].msg) do
				AB_Print(format(AB_STATUS_MSG,x,AB_Data[i].msg[x]))
			end
			AB_Print(format(AB_STATUS_CHAN,AB_Data[i].chan))
			if AB_Data[i].password ~= "" then
				AB_Print(format(AB_STATUS_PWD,AB_Data[i].password))
			end
			AB_Print(format(AB_STATUS_INTERVAL,AB_Data[i].interval))
			if AB_Data[i].enabled then
				AB_Print(format(AB_STATUS_STATE,AB_TRUE))
			else
				AB_Print(format(AB_STATUS_STATE,AB_FALSE))
			end
			if id == AB_Data[i].id then
				break
			end
		end
	end
end

function AB_IsEnabled(x)
	local id = strtrim(strupper(x))
	for i=1,#(AB_Data) do
		if AB_Data[i] and AB_Data[i].id == id then
			return AB_Data[i].enabled
		end
	end
	return nil
end

function AB_Print(x)
	DEFAULT_CHAT_FRAME:AddMessage(x)
end

function AB_GetChannels(x)
	supportedChannels = { GetChannelList() }
	local otherChannels = { "Say", "Yell", "Guild", "Party", "Raid", "Officer" }
	for i=1,#(otherChannels) do
		tinsert(supportedChannels,otherChannels[i])
		tinsert(supportedChannels,otherChannels[i])
	end
end

-- Interface Functions

function AB_New_Group()
	selectedID = 0
	selectedMsg = 0
	selectedChan = 0
	selectedRotation = RR
	ABEditFrameID:SetText("")
	ABEditFrameMessage:SetText("")
	ABEditFrameInterval:SetText("180")
	ABEditFrameChannel:SetText("")
	ABEditFramePassword:SetText("")
	ABEditFrameEnable:SetChecked(true)
	ABEditFrameRotation:SetText(AB_ROTATION[selectedRotation])
	ABEditFrameID:SetFocus()
	ABEditFrame:Show()
	ABEditFrameButtonSave:Disable()
end

function AB_New_Message()
	if AB_Data[selectedID] then
		selectedMsg = #(AB_Data[selectedID].msg) + 1
	end
	ABEditFrameMessage:SetText("")
	ABEditFrameMessage:SetFocus()
	ABEditFrameButtonSave:Disable()
end

function AB_Edit(x, y)
	local id = strtrim(strupper(x))
	for i=1,#(AB_Data) do
		if AB_Data[i] and AB_Data[i].id == id then
			ABEditFrameID:SetText(AB_Data[i].id)
			if AB_Data[i].msg[y] then
				selectedMsg = y
				selectedID = i
				ABEditFrameMessage:SetText(AB_Data[i].msg[y])
			else
				ABEditFrameMessage:SetText("")
			end
			selectedRotation = AB_Data[i].rotation
			ABEditFrameRotation:SetText(AB_ROTATION[selectedRotation])
			ABEditFrameInterval:SetText(AB_Data[i].interval)
			ABEditFrameChannel:SetText(AB_Data[i].chan)
			ABEditFramePassword:SetText(AB_Data[i].password)
			ABEditFrameEnable:SetChecked(AB_Data[i].enabled)
			ABEditFrameButtonSave:Enable()
			ABEditFrameMessage:SetFocus()
			ABEditFrame:Show()
			ABEditFrameButtonSave:Disable()
			return
		end
	end
	AB_New_Group()
	ABEditFrameID:SetText(id)
	ABEditFrameMessage:SetFocus()
	ABEditFrameButtonSave:Disable()
end

function AB_GetTooltip()
	local tip = ""
	if not AB_On then
		return AB_TOOLTIP_OFF
	end
	if AB_IsPaused() then
		return AB_TOOLTIP_PAUSED
	end
	for i=1,#(AB_Data) do
		if AB_Data[i] and AB_Data[i].enabled then
			if tip ~= "" then
				tip = tip.."\n"
			end
			tip = tip.. format(AB_TOOLTIP_STATUS,AB_Data[i].id,floor(AB_Data[i].ttl-GetTime()))
		end
	end
	if tip == "" then
		tip = AB_TOOLTIP_NONE
	end
	return tip
end

function AB_IsPaused()
	if UnitIsDND("player") or UnitIsAFK("player") or isBG == true or (UnitIsPVP("player") and UnitAffectingCombat("player")) then
		return true
	end
	return false
end

function AB_GetButtonText()
	if AB_On then
		return AB_ON
	else
		return AB_OFF
	end
end

function AB_DropDownMenu_OnLoad()
	UIDropDownMenu_Initialize(this:GetParent(),AB_DropDownMenu_Init)
	UIDropDownMenu_SetAnchor(0,0,this:GetParent(),"TOPRIGHT",this:GetName(),"BOTTOMRIGHT")
end

function AB_DropDownMenu_Init()
	local info = {
		text = AB_NEW,
		value = 0,
		func = AB_New_Group,
		notCheckable=1
	}
	UIDropDownMenu_AddButton(info)
	local id = strupper(strtrim(ABEditFrameID:GetText()))
	if id ~= "" then
		selectedID = AB_InList(id)
	end
	for i=1,#(AB_Data) do
		info = {
			text = AB_Data[i].id,
			value = i,
			func = AB_ListSelect
		}
		if i == selectedID then 
			info.checked = 1
		end
		UIDropDownMenu_AddButton(info)
	end
end

function AB_ListSelect()
	if this.value and AB_Data[this.value] then
		selectedID = this.value
		AB_Edit(AB_Data[selectedID].id, 1)
		ABEditFrameMessage:SetFocus()
		AB_Msg_DDM_OnLoad()
		AB_Rotation_DDM_OnLoad()
	end
end

function AB_InList(x)
	for i=1,#(AB_Data) do
		if AB_Data[i] and x == AB_Data[i].id then
			return i
		end
	end
	return 0
end

function AB_Msg_DDM_OnLoad()
	UIDropDownMenu_Initialize(this:GetParent(),AB_Msg_DDM_Init)
	UIDropDownMenu_SetAnchor(0,0,this:GetParent(),"TOPRIGHT",this:GetName(),"BOTTOMRIGHT")
end

function AB_Msg_DDM_Init()
	local info = {
		text = AB_NEW,
		value = 0,
		func = AB_New_Message,
		notCheckable=1
	}
	UIDropDownMenu_AddButton(info)
	if (AB_Data[selectedID]) then
		for i=1,#(AB_Data[selectedID].msg) do
			info = {
				text = i,
				value = i,
				func = AB_Msg_DDM_ListSelect
			}
			if i == selectedMsg then 
				info.checked = 1
			end
			UIDropDownMenu_AddButton(info)
		end
	end
end

function AB_Msg_DDM_ListSelect()
	if this.value and AB_Data[selectedID] and AB_Data[selectedID].msg[this.value] then
		selectedMsg = this.value
		ABEditFrameMessage:SetText(AB_Data[selectedID].msg[selectedMsg])
		ABEditFrameMessage:SetFocus()
	end
end

function AB_Msg_DDM_InList(x)
	if AB_Data[selectedID] and AB_Data[selectedID].msg[x] then
		return x
	end
	return 0
end

function AB_Chan_DDM_OnLoad()
	UIDropDownMenu_Initialize(this:GetParent(),AB_Chan_DDM_Init)
	UIDropDownMenu_SetAnchor(0,0,this:GetParent(),"TOPRIGHT",this:GetName(),"BOTTOMRIGHT")
end

function AB_Chan_DDM_Init()
	AB_GetChannels()
	local info = {}
	local id = ABEditFrameChannel:GetText()
	if id ~= "" then
		selectedChan = AB_Chan_DDM_InList(id)
	end
	for i=2,#(supportedChannels),2 do
		info = {
			text = supportedChannels[i],
			value = i-1,
			func = AB_Chan_DDM_ListSelect
		}
		if i == selectedChan then
			info.checked = 1
		end
		UIDropDownMenu_AddButton(info)
	end
end

function AB_Chan_DDM_ListSelect()
	if this.value and supportedChannels[this.value] then
		ABEditFrameChannel:SetText(supportedChannels[this.value])
		ABEditFrameChannel:SetFocus()
	end
end

function AB_Chan_DDM_InList(x)
	for i=2,#(supportedChannels),2 do
		if supportedChannels[i] and ( tonumber(x) == supportedChannels[i-1] or x == supportedChannels[i]) then
			return i
		end
	end
	return 0
end

function AB_Rotation_DDM_OnLoad()
	UIDropDownMenu_Initialize(this:GetParent(),AB_Rotation_DDM_Init)
	UIDropDownMenu_SetAnchor(0,0,this:GetParent(),"TOPRIGHT",this:GetName(),"BOTTOMRIGHT")
end

function AB_Rotation_DDM_Init()
	local info = {}
	local id = ABEditFrameRotation:GetText()
	if id ~= "" then
		selectedRotation = AB_Rotation_DDM_InList(id)
	end
	for i=1,#(AB_ROTATION) do
		info = {
			text = AB_ROTATION[i],
			value = i,
			func = AB_Rotation_DDM_ListSelect
		}
		if i == selectedRotation then
			info.checked = 1
		end
		UIDropDownMenu_AddButton(info)
	end
end

function AB_Rotation_DDM_ListSelect()
	if this.value and AB_ROTATION[this.value] then
		selectedRotation = this.value
		ABEditFrameRotation:SetText(AB_ROTATION[this.value])
	end
end

function AB_Rotation_DDM_InList(x)
	for i=1,#(AB_ROTATION) do
		if AB_ROTATION[i] and x == AB_ROTATION[i] then
			return i
		end
	end
	return 0
end

function AB_ChatEdit_InsertLink(x)
	if AB_EditBoxInFocus then
		if (strlen(AB_EditBoxInFocus:GetText()) + strlen(x)) < 255 then
			AB_EditBoxInFocus:Insert(x)
		end
		return true
	end
	return ORG_ChatEdit_InsertLink(x)
end

function AB_SetItemRef(link, text, button)
	if strsub(link, 1, 6) ~= "player" and IsShiftKeyDown() and ( not ChatFrameEditBox:IsVisible() ) then
		if AB_EditBoxInFocus then
			AB_EditBoxInFocus:Insert(text)
			return nil
		end
	end
	return ORG_SetItemRef(link, text, button)
end

function AB_ATSW_SkillIcon_OnClick(self,button,down)
	if IsControlKeyDown() then
		DressUpItemLink(ATSW_GetTradeSkillItemLink(ATSWFrame.selectedSkill))
	elseif IsShiftKeyDown() then
		if AB_EditBoxInFocus then
			AB_EditBoxInFocus:Insert(ATSW_GetTradeSkillItemLink(ATSWFrame.selectedSkill))
		elseif ChatFrameEditBox:IsVisible() then
			ChatFrameEditBox:Insert(ATSW_GetTradeSkillItemLink(ATSWFrame.selectedSkill))
		end
	end
end

function AB_Strip(x, y)
	local p, n = strfind(strlower(strtrim(x)), strlower(y), 1, true)
	if p ~= nil and p == 1 and n ~= nil then
		return strtrim(strsub(x, n+1, strlen(x)))
	end
	return nil
end

function AB_Skill_OnUpdate()
	for i=1, GetNumSkillLines() do
		local skillName, header, _, skillRank = GetSkillLineInfo(i)
		if skillName and not header then
			if skillName == AB_ALCHEMY then
				AB_UpdateVariable("!alch!",skillRank)
			elseif skillName == AB_BLACKSMITHING then
				AB_UpdateVariable("!bs!",skillRank)
			elseif skillName == AB_ENCHANTING then
				AB_UpdateVariable("!enc!",skillRank)
			elseif skillName == AB_ENGINEERING then
				AB_UpdateVariable("!eng!",skillRank)
			elseif skillName == AB_LEATHERWORKING then
				AB_UpdateVariable("!lw!",skillRank)
			elseif skillName == AB_JEWELCRAFTING then
				AB_UpdateVariable("!jc!",skillRank)
			elseif skillName == AB_TAILORING then
				AB_UpdateVariable("!tail!",skillRank)
			elseif skillName == AB_COOKING then
				AB_UpdateVariable("!cook!",skillRank)
			elseif skillName == AB_FIRSTAID then
				AB_UpdateVariable("!aid!",skillRank)
			elseif skillName == AB_FISHING then
				AB_UpdateVariable("!fish!",skillRank)
			elseif skillName == AB_POISONS then
				AB_UpdateVariable("!poi!",skillRank)
			elseif skillName == AB_LOCKPICKING then
				AB_UpdateVariable("!lp!",skillRank)
			end
		end
	end
end

function AB_DeleteVariable(x,y)
	for i=1, #(AB_Variables) do
		if AB_Variables[i][1] == x then
			tremove(AB_Variables,i)
		end
	end
end

function AB_PrintVariables()
	for i=1, #(AB_Variables) do
		AB_Print(AB_Variables[i][1].."="..AB_Variables[i][2])
	end
end

function AB_UpdateVariable(x,y)
	for i=1, #(AB_Variables) do
		if AB_Variables[i][1] == x then
			AB_Variables[i][2] = y
			return
		end
	end
	tinsert(AB_Variables, {x,y})
end

function AB_GetVariableValue(x)
	for i=1, #(AB_Variables) do
		local var, value = AB_Variables[i][1],AB_Variables[i][2]
		if x == var and value then
			if strfind(value, "return ",1,true) then
				local func, err = loadstring(value)
				if func then
					value = func()
				end
			end
			return value
		end
	end
	return nil
end

function AB_ResetVariables()
	AB_Variables = {
		{"!loc!", "local x, y = GetPlayerMapPosition(\"player\");return format(\"(%s,%s)\",floor(x*100),floor(y*100))" },
		{"!lvl!", "return UnitLevel(\"player\")" },
		{"!gr!", "local gn, gr = GetGuildInfo(\"player\");if gn then return gr else return \"\" end" },
		{"!gn!", "local gn = GetGuildInfo(\"player\");if gn then return gn else return \"\" end" },
		{"!z!", "return GetZoneText()" },
		{"!sz!", "return GetMinimapZoneText()" },
		{"!alch!", 0},
		{"!bs!", 0},
		{"!enc!", 0},
		{"!eng!", 0},
		{"!lw!", 0},
		{"!jc!", 0},
		{"!tail!", 0},
		{"!cook!", 0},
		{"!aid!", 0},
		{"!fish!", 0},
		{"!poi!", 0},
		{"!lp!", 0},
	}
	AB_Skill_OnUpdate()
end

function AB_ReplaceVariables(x)
	for i=1, #(AB_Variables) do
		local var, value = AB_Variables[i][1],AB_Variables[i][2]
		if strfind(x,var,1,true) and value then
			if strfind(value, "return ",1,true) then
				local func, err = loadstring(value)
				if func then
					value = func()
				end
			end
			if value == nil then
				value = ""
			end
			x = gsub(x,var,value)
		end
	end
	return x
end

function AB_ValidateForm()
	if strtrim(ABEditFrameID:GetText()) == "" or strtrim(ABEditFrameChannel:GetText()) == "" or strtrim(ABEditFrameMessage:GetText()) == "" or ABEditFrameInterval:GetNumber() == 0 then
		ABEditFrameButtonSave:Disable()
		return false
	else
		ABEditFrameButtonSave:Enable()
		return true
	end
end

function AB_SaveForm()
	local msg = ABEditFrameMessage:GetText()
	local msgs = { strsplit("\n", msg) }
	for i=1, #(msgs) do
		if strlen(msgs[i]) > 255 then
			AB_Print(AB_ERR_MSGTOOLONG .. " (line "..i..")")
			return
		end
	end
	local enabled = false
	if ABEditFrameEnable:GetChecked() then
		enabled = true
	end
	ABEditFrameEnable:SetChecked(enabled)
	local id = strtrim(ABEditFrameID:GetText())
	local password = ABEditFramePassword:GetText()
	local interval = ABEditFrameInterval:GetNumber()
	local channel = ABEditFrameChannel:GetText()
	AB_Insert(id, selectedMsg, msg,channel,password,interval,nil,enabled,selectedRotation)
	ABEditFrameButtonSave:Disable()
end